import * as tslib_1 from "tslib";
import * as React from 'react';
import { brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { ExpansionPanel } from './ExpansionPanel';
import { IgrOnExpandedEventArgs } from './igr-on-expanded-event-args';
import { IgrOnCollapsedEventArgs } from './igr-on-collapsed-event-args';
var IgrExpansionPanel = /** @class */ /*@__PURE__*/ (function (_super) {
    tslib_1.__extends(IgrExpansionPanel, _super);
    function IgrExpansionPanel(props) {
        var _this = _super.call(this, props) || this;
        _this._reactRenderer = null;
        _this._implementation = null;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._onCollapsed = null;
        _this._onCollapsed_wrapped = null;
        _this._onExpanded = null;
        _this._onExpanded_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._getMainRef = _this._getMainRef.bind(_this);
        _this._getHeaderRef = _this._getHeaderRef.bind(_this);
        _this._getContentAreaOuterRef = _this._getContentAreaOuterRef.bind(_this);
        _this._getContentAreaInnerRef = _this._getContentAreaInnerRef.bind(_this);
        if (props) {
            _this.initializeProperties();
        }
        _this._initialized = true;
        return _this;
    }
    Object.defineProperty(IgrExpansionPanel.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            this._height = value;
            if (this._elRef) {
                this._elRef.style.height = value;
                this.i.notifySizeChanged();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            this._width = value;
            if (this._elRef) {
                this._elRef.style.height = value;
                this.i.notifySizeChanged();
            }
        },
        enumerable: true,
        configurable: true
    });
    IgrExpansionPanel.prototype._getMainRef = function (ref) {
        this._elRef = ref;
        this.verifyReferences();
    };
    IgrExpansionPanel.prototype._getHeaderRef = function (ref) {
        this._headerRef = ref;
        this.verifyReferences();
    };
    IgrExpansionPanel.prototype._getContentAreaOuterRef = function (ref) {
        this._contentAreaOuterRef = ref;
        this.verifyReferences();
    };
    IgrExpansionPanel.prototype._getContentAreaInnerRef = function (ref) {
        this._contentAreaInnerRef = ref;
        this.verifyReferences();
    };
    IgrExpansionPanel.prototype.verifyReferences = function () {
        if (this._elRef &&
            this._headerRef &&
            this._contentAreaOuterRef &&
            this._contentAreaInnerRef) {
            this.requestRender = this.requestRender.bind(this);
            this._portalManager = new PortalManager("iconContent", this.requestRender);
            var ren = new ReactRenderer(this._elRef, document, true, {}, this._portalManager);
            // render needs to fire again after the portalManager is created. 
            this.requestRender();
            this._reactRenderer = ren;
            this.i.provideRenderer(this._reactRenderer);
            var headerWrapper = this._reactRenderer.getWrapper(this._headerRef);
            this.i.provideHeader(headerWrapper);
            var contentOuterWrapper = this._reactRenderer.getWrapper(this._contentAreaOuterRef);
            this.i.provideContentAreaOuter(contentOuterWrapper);
            var contentInnerWrapper = this._reactRenderer.getWrapper(this._contentAreaInnerRef);
            this.i.provideContentAreaInner(contentInnerWrapper);
        }
    };
    IgrExpansionPanel.prototype.render = function () {
        var propChildren = this.props.children;
        var children = [];
        React.Children.forEach(propChildren, function (ch) {
            children.push(React.cloneElement(ch));
        });
        var divChildren = [];
        var header = React.createElement("div", {
            key: 'expansionPanelHeaderArea',
            ref: this._getHeaderRef
        });
        var contentInner = React.createElement("div", {
            key: 'expansionPanelContentAreaInner',
            ref: this._getContentAreaInnerRef,
            children: children,
        });
        var contentOuter = React.createElement("div", {
            key: 'expansionPanelContentAreaOuter',
            ref: this._getContentAreaOuterRef,
            children: contentInner
        });
        divChildren.push(header);
        divChildren.push(contentOuter);
        if (this._portalManager)
            this._portalManager.onRender(divChildren);
        var div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-expansion-panel igr-expansion-panel",
            children: divChildren
        });
        return div;
    };
    IgrExpansionPanel.prototype.requestRender = function () {
        if (this._initialized)
            this.setState({});
    };
    IgrExpansionPanel.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        var e_1, _a;
        var mod = getModifiedProps(this.props, nextProps);
        try {
            for (var _b = tslib_1.__values(Object.keys(mod)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var p = _c.value;
                if (isValidProp(this, p)) {
                    this[p] = mod[p];
                }
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return true;
    };
    IgrExpansionPanel.prototype.initializeProperties = function () {
        var e_2, _a;
        try {
            for (var _b = tslib_1.__values(Object.keys(this.props)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var p = _c.value;
                if (isValidProp(this, p)) {
                    this[p] = this.props[p];
                }
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
    };
    // supports angular themes or custom properties set in CSS
    IgrExpansionPanel.prototype.updateStyle = function () {
        this._styling(this._elRef, this);
    };
    IgrExpansionPanel.prototype.componentWillUnmount = function () {
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    };
    IgrExpansionPanel.prototype.componentDidMount = function () {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.initializeContent();
    };
    IgrExpansionPanel.prototype.initializeContent = function () {
        this._styling(this._elRef, this);
        this.updateStyle();
    };
    IgrExpansionPanel.prototype.createImplementation = function () {
        return new ExpansionPanel();
    };
    Object.defineProperty(IgrExpansionPanel.prototype, "i", {
        get: function () {
            return this._implementation;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "caption", {
        /**
     * Gets / sets the caption to display in the header area.
    */
        get: function () {
            return this.i.ab;
        },
        set: function (v) {
            this.i.ab = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "captionTextColor", {
        /**
         * Gets or sets the color to use for the header caption text.
        */
        get: function () {
            return brushToString(this.i.bq);
        },
        set: function (v) {
            this.i.bq = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualCaptionTextColor", {
        /**
         * Gets or sets the color to use for the actual header caption text.
        */
        get: function () {
            return brushToString(this.i.bj);
        },
        set: function (v) {
            this.i.bj = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "captionCollapsedTextColor", {
        /**
         * Gets or sets the color to use for the header caption text when the panel is collapsed.
        */
        get: function () {
            return brushToString(this.i.bo);
        },
        set: function (v) {
            this.i.bo = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "captionExpandedTextColor", {
        /**
         * Gets or sets the color to use for the header caption text when the panel is expanded.
        */
        get: function () {
            return brushToString(this.i.bp);
        },
        set: function (v) {
            this.i.bp = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "description", {
        /**
         * Gets / sets the description to display in the header area.
        */
        get: function () {
            return this.i.ah;
        },
        set: function (v) {
            this.i.ah = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "descriptionTextColor", {
        /**
         * Gets or sets the color to use for the header description text.
        */
        get: function () {
            return brushToString(this.i.bz);
        },
        set: function (v) {
            this.i.bz = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualDescriptionTextColor", {
        /**
         * Gets or sets the color to use for the actual header description text.
        */
        get: function () {
            return brushToString(this.i.bk);
        },
        set: function (v) {
            this.i.bk = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "descriptionCollapsedTextColor", {
        /**
         * Gets or sets the color to use for the header description text when the panel is collapsed.
        */
        get: function () {
            return brushToString(this.i.bx);
        },
        set: function (v) {
            this.i.bx = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "descriptionExpandedTextColor", {
        /**
         * Gets or sets the color to use for the header description text when the panel is expanded.
        */
        get: function () {
            return brushToString(this.i.by);
        },
        set: function (v) {
            this.i.by = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "elevation", {
        /**
         * Gets or sets the elevation to use for the button regardless of type.
        */
        get: function () {
            return this.i.s;
        },
        set: function (v) {
            this.i.s = +v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualElevation", {
        /**
         * Gets or sets the elevation of the border shadow
        */
        get: function () {
            return this.i.q;
        },
        set: function (v) {
            this.i.q = +v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "expanded", {
        /**
         * Gets / sets the expanded state of the panel.
        */
        get: function () {
            return this.i.n;
        },
        set: function (v) {
            this.i.n = ensureBool(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "headerBackgroundColor", {
        /**
         * Gets or sets the color to use for the background of the header.
        */
        get: function () {
            return brushToString(this.i.b6);
        },
        set: function (v) {
            this.i.b6 = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualHeaderBackgroundColor", {
        /**
         * Gets or sets the color to use for the actual background of the header.
        */
        get: function () {
            return brushToString(this.i.bl);
        },
        set: function (v) {
            this.i.bl = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "headerCollapsedBackgroundColor", {
        /**
         * Gets or sets the color to use for the background of the header when the panel is expanded.
        */
        get: function () {
            return brushToString(this.i.b7);
        },
        set: function (v) {
            this.i.b7 = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "headerExpandedBackgroundColor", {
        /**
         * Gets or sets the color to use for the background of the header when the panel is expanded.
        */
        get: function () {
            return brushToString(this.i.b8);
        },
        set: function (v) {
            this.i.b8 = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualUmbraShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.bn);
        },
        set: function (v) {
            this.i.bn = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualPenumbraShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.bm);
        },
        set: function (v) {
            this.i.bm = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "actualAmbientShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.bi);
        },
        set: function (v) {
            this.i.bi = stringToBrush(v);
        },
        enumerable: true,
        configurable: true
    });
    IgrExpansionPanel.prototype.findByName = function (name) {
        return null;
    };
    Object.defineProperty(IgrExpansionPanel.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: true,
        configurable: true
    });
    IgrExpansionPanel.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgrExpansionPanel.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ExpansionPanel");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    /**
 * Collapses the contents of the expansion panel.

*/
    IgrExpansionPanel.prototype.collapse = function () {
        this.i.ar();
    };
    /**
     * Expands the contents of the expansion panel.
    
    */
    IgrExpansionPanel.prototype.expand = function () {
        this.i.au();
    };
    /**
     * Toggles the expanded state of the expansion panel.
    
    */
    IgrExpansionPanel.prototype.toggle = function () {
        this.i.a2();
    };
    Object.defineProperty(IgrExpansionPanel.prototype, "onCollapsed", {
        /**
         * Fires when the expansion panel is collapsed.
        */
        get: function () {
            return this._onCollapsed;
        },
        set: function (ev) {
            var _this = this;
            if (this._onCollapsed_wrapped !== null) {
                this.i.onCollapsed = delegateRemove(this.i.onCollapsed, this._onCollapsed_wrapped);
                this._onCollapsed_wrapped = null;
                this._onCollapsed = null;
            }
            this._onCollapsed = ev;
            this._onCollapsed_wrapped = function (o, e) {
                var outerArgs = new IgrOnCollapsedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnCollapsed) {
                    _this.beforeOnCollapsed(_this, outerArgs);
                }
                if (_this._onCollapsed) {
                    _this._onCollapsed(_this, outerArgs);
                }
            };
            this.i.onCollapsed = delegateCombine(this.i.onCollapsed, this._onCollapsed_wrapped);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgrExpansionPanel.prototype, "onExpanded", {
        get: function () {
            return this._onExpanded;
        },
        set: function (ev) {
            var _this = this;
            if (this._onExpanded_wrapped !== null) {
                this.i.onExpanded = delegateRemove(this.i.onExpanded, this._onExpanded_wrapped);
                this._onExpanded_wrapped = null;
                this._onExpanded = null;
            }
            this._onExpanded = ev;
            this._onExpanded_wrapped = function (o, e) {
                var outerArgs = new IgrOnExpandedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnExpanded) {
                    _this.beforeOnExpanded(_this, outerArgs);
                }
                if (_this._onExpanded) {
                    _this._onExpanded(_this, outerArgs);
                }
            };
            this.i.onExpanded = delegateCombine(this.i.onExpanded, this._onExpanded_wrapped);
        },
        enumerable: true,
        configurable: true
    });
    return IgrExpansionPanel;
}(React.Component));
export { IgrExpansionPanel };
//# sourceMappingURL=igr-expansion-panel.js.map
